==> dt-common-0.5.5.tgz: package/models/cavern_maps/StairsFactory.js <==
const Balance = require('../../constants/Balance');

const StairsFactory = function(tiles, entrance, cavernsLevel, max_stairs = 1) {
  const stairs = [];

  // do stairs
  const makeStairs = (x, y, direction, isEntrance) => {
    return {
      stairs: true,
      x,
      y,
      direction,
      isEntrance,
    };
  };

  const candidates = [];
  for (let i = 0; i < tiles.length; ++i) {
    if (!tiles[i].occupied) {
      candidates.push(tiles[i]);
    }
  }

  // make any stairs the player has come up or down from
  if (entrance === 'UP' || entrance === 'DOWN') {
    const randomCandidate = candidates[
      Math.floor(Math.random() * candidates.length)
    ];
    randomCandidate.occupied = true;
    stairs.push(
      makeStairs(randomCandidate.x, randomCandidate.y, entrance, true),
    );
  }

  // random stair generation
  const num_stairs = Math.ceil(Math.random() * max_stairs);
  for (let j = 0; j < num_stairs; ++j) {
    const randomCandidate = candidates[ Math.floor(Math.random()*candidates.length) ];
    randomCandidate.occupied = true;

    if (Math.random() < Balance.CHANCE_TO_SPAWN_STAIRS / num_stairs) {
      switch (Math.round(Math.random())) {
        case 0:
          stairs.push(
            makeStairs(randomCandidate.x, randomCandidate.y, 'DOWN', false),
          );
          break;
        case 1:
          if (cavernsLevel > 1) {
            stairs.push(
              makeStairs(randomCandidate.x, randomCandidate.y, 'UP', false),
            );
          }
          break;
      }
    }
  }

  return stairs;
};
module.exports = StairsFactory;

==> dt-common-0.5.12.tgz: package/models/cavern_maps/StairsFactory.js <==
const Balance = require('../../constants/Balance');

const StairsFactory = function(tiles, entrance, cavernsLevel, max_stairs = 1) {
  const stairs = [];

  // do stairs
  const makeStairs = (x, y, direction, isEntrance) => {
    return {
      stairs: true,
      x,
      y,
      direction,
      isEntrance,
    };
  };

  const candidates = [];
  for (let i = 0; i < tiles.length; ++i) {
    if (!tiles[i].occupied) {
      candidates.push(tiles[i]);
    }
  }

  // make any stairs the player has come up or down from
  if (entrance === 'UP' || entrance === 'DOWN') {
    const randomCandidate = candidates[
      Math.floor(Math.random() * candidates.length)
    ];
    randomCandidate.occupied = true;
    stairs.push(
      makeStairs(randomCandidate.x, randomCandidate.y, entrance, true),
    );
  }

  // random stair generation
  const num_stairs = Math.ceil(Math.random() * max_stairs);
  for (let j = 0; j < num_stairs; ++j) {
    const randomCandidate = candidates[ Math.floor(Math.random()*candidates.length) ];
    randomCandidate.occupied = true;

    if (Math.random() < 1) { // Balance.CHANCE_TO_SPAWN_STAIRS) {
      switch (Math.round(Math.random())) {
        case 0:
          stairs.push(
            makeStairs(randomCandidate.x, randomCandidate.y, 'DOWN', false),
          );
          break;
        case 1:
          if (cavernsLevel > 1) {
            stairs.push(
              makeStairs(randomCandidate.x, randomCandidate.y, 'UP', false),
            );
          }
          break;
      }
    }
  }

  return stairs;
};
module.exports = StairsFactory;
